/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.curator.framework.recipes.queue;

class TestQueueItem implements Comparable<TestQueueItem>
{
    final String    str;

    TestQueueItem(String str)
    {
        this.str = str;
    }

    @Override
    public int compareTo(TestQueueItem rhs)
    {
        if ( this == rhs )
        {
            return 0;
        }

        int         val = Integer.parseInt(str);
        int         rhsVal = Integer.parseInt(rhs.str);
        int         diff = val - rhsVal;
        return (diff < 0) ? -1 : ((diff > 0) ? 1 : 0);
    }

    @Override
    public boolean equals(Object o)
    {
        if ( this == o )
        {
            return true;
        }
        if ( o == null || getClass() != o.getClass() )
        {
            return false;
        }

        TestQueueItem that = (TestQueueItem)o;

        return str.equals(that.str);

    }

    @Override
    public int hashCode()
    {
        return str.hashCode();
    }
}
