/**
 * Autogenerated by Thrift Compiler (0.9.1)
 *
 * DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
 *  @generated
 */
package org.apache.curator.generated;

import org.apache.thrift.scheme.IScheme;
import org.apache.thrift.scheme.SchemeFactory;
import org.apache.thrift.scheme.StandardScheme;

import org.apache.thrift.scheme.TupleScheme;
import org.apache.thrift.protocol.TTupleProtocol;
import org.apache.thrift.protocol.TProtocolException;
import org.apache.thrift.EncodingUtils;
import org.apache.thrift.TException;
import org.apache.thrift.async.AsyncMethodCallback;
import org.apache.thrift.server.AbstractNonblockingServer.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.EnumMap;
import java.util.Set;
import java.util.HashSet;
import java.util.EnumSet;
import java.util.Collections;
import java.util.BitSet;
import java.nio.ByteBuffer;
import java.util.Arrays;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LeaderResult implements org.apache.thrift.TBase<LeaderResult, LeaderResult._Fields>, java.io.Serializable, Cloneable, Comparable<LeaderResult> {
  private static final org.apache.thrift.protocol.TStruct STRUCT_DESC = new org.apache.thrift.protocol.TStruct("LeaderResult");

  private static final org.apache.thrift.protocol.TField PROJECTION_FIELD_DESC = new org.apache.thrift.protocol.TField("projection", org.apache.thrift.protocol.TType.STRUCT, (short)1);
  private static final org.apache.thrift.protocol.TField IS_LEADER_FIELD_DESC = new org.apache.thrift.protocol.TField("isLeader", org.apache.thrift.protocol.TType.BOOL, (short)2);

  private static final Map<Class<? extends IScheme>, SchemeFactory> schemes = new HashMap<Class<? extends IScheme>, SchemeFactory>();
  static {
    schemes.put(StandardScheme.class, new LeaderResultStandardSchemeFactory());
    schemes.put(TupleScheme.class, new LeaderResultTupleSchemeFactory());
  }

  public LeaderProjection projection; // required
  public boolean isLeader; // required

  /** The set of fields this struct contains, along with convenience methods for finding and manipulating them. */
  public enum _Fields implements org.apache.thrift.TFieldIdEnum {
    PROJECTION((short)1, "projection"),
    IS_LEADER((short)2, "isLeader");

    private static final Map<String, _Fields> byName = new HashMap<String, _Fields>();

    static {
      for (_Fields field : EnumSet.allOf(_Fields.class)) {
        byName.put(field.getFieldName(), field);
      }
    }

    /**
     * Find the _Fields constant that matches fieldId, or null if its not found.
     */
    public static _Fields findByThriftId(int fieldId) {
      switch(fieldId) {
        case 1: // PROJECTION
          return PROJECTION;
        case 2: // IS_LEADER
          return IS_LEADER;
        default:
          return null;
      }
    }

    /**
     * Find the _Fields constant that matches fieldId, throwing an exception
     * if it is not found.
     */
    public static _Fields findByThriftIdOrThrow(int fieldId) {
      _Fields fields = findByThriftId(fieldId);
      if (fields == null) throw new IllegalArgumentException("Field " + fieldId + " doesn't exist!");
      return fields;
    }

    /**
     * Find the _Fields constant that matches name, or null if its not found.
     */
    public static _Fields findByName(String name) {
      return byName.get(name);
    }

    private final short _thriftId;
    private final String _fieldName;

    _Fields(short thriftId, String fieldName) {
      _thriftId = thriftId;
      _fieldName = fieldName;
    }

    public short getThriftFieldId() {
      return _thriftId;
    }

    public String getFieldName() {
      return _fieldName;
    }
  }

  // isset id assignments
  private static final int __ISLEADER_ISSET_ID = 0;
  private byte __isset_bitfield = 0;
  public static final Map<_Fields, org.apache.thrift.meta_data.FieldMetaData> metaDataMap;
  static {
    Map<_Fields, org.apache.thrift.meta_data.FieldMetaData> tmpMap = new EnumMap<_Fields, org.apache.thrift.meta_data.FieldMetaData>(_Fields.class);
    tmpMap.put(_Fields.PROJECTION, new org.apache.thrift.meta_data.FieldMetaData("projection", org.apache.thrift.TFieldRequirementType.DEFAULT, 
        new org.apache.thrift.meta_data.StructMetaData(org.apache.thrift.protocol.TType.STRUCT, LeaderProjection.class)));
    tmpMap.put(_Fields.IS_LEADER, new org.apache.thrift.meta_data.FieldMetaData("isLeader", org.apache.thrift.TFieldRequirementType.DEFAULT, 
        new org.apache.thrift.meta_data.FieldValueMetaData(org.apache.thrift.protocol.TType.BOOL)));
    metaDataMap = Collections.unmodifiableMap(tmpMap);
    org.apache.thrift.meta_data.FieldMetaData.addStructMetaDataMap(LeaderResult.class, metaDataMap);
  }

  public LeaderResult() {
  }

  public LeaderResult(
    LeaderProjection projection,
    boolean isLeader)
  {
    this();
    this.projection = projection;
    this.isLeader = isLeader;
    setIsLeaderIsSet(true);
  }

  /**
   * Performs a deep copy on <i>other</i>.
   */
  public LeaderResult(LeaderResult other) {
    __isset_bitfield = other.__isset_bitfield;
    if (other.isSetProjection()) {
      this.projection = new LeaderProjection(other.projection);
    }
    this.isLeader = other.isLeader;
  }

  public LeaderResult deepCopy() {
    return new LeaderResult(this);
  }

  @Override
  public void clear() {
    this.projection = null;
    setIsLeaderIsSet(false);
    this.isLeader = false;
  }

  public LeaderProjection getProjection() {
    return this.projection;
  }

  public LeaderResult setProjection(LeaderProjection projection) {
    this.projection = projection;
    return this;
  }

  public void unsetProjection() {
    this.projection = null;
  }

  /** Returns true if field projection is set (has been assigned a value) and false otherwise */
  public boolean isSetProjection() {
    return this.projection != null;
  }

  public void setProjectionIsSet(boolean value) {
    if (!value) {
      this.projection = null;
    }
  }

  public boolean isIsLeader() {
    return this.isLeader;
  }

  public LeaderResult setIsLeader(boolean isLeader) {
    this.isLeader = isLeader;
    setIsLeaderIsSet(true);
    return this;
  }

  public void unsetIsLeader() {
    __isset_bitfield = EncodingUtils.clearBit(__isset_bitfield, __ISLEADER_ISSET_ID);
  }

  /** Returns true if field isLeader is set (has been assigned a value) and false otherwise */
  public boolean isSetIsLeader() {
    return EncodingUtils.testBit(__isset_bitfield, __ISLEADER_ISSET_ID);
  }

  public void setIsLeaderIsSet(boolean value) {
    __isset_bitfield = EncodingUtils.setBit(__isset_bitfield, __ISLEADER_ISSET_ID, value);
  }

  public void setFieldValue(_Fields field, Object value) {
    switch (field) {
    case PROJECTION:
      if (value == null) {
        unsetProjection();
      } else {
        setProjection((LeaderProjection)value);
      }
      break;

    case IS_LEADER:
      if (value == null) {
        unsetIsLeader();
      } else {
        setIsLeader((Boolean)value);
      }
      break;

    }
  }

  public Object getFieldValue(_Fields field) {
    switch (field) {
    case PROJECTION:
      return getProjection();

    case IS_LEADER:
      return Boolean.valueOf(isIsLeader());

    }
    throw new IllegalStateException();
  }

  /** Returns true if field corresponding to fieldID is set (has been assigned a value) and false otherwise */
  public boolean isSet(_Fields field) {
    if (field == null) {
      throw new IllegalArgumentException();
    }

    switch (field) {
    case PROJECTION:
      return isSetProjection();
    case IS_LEADER:
      return isSetIsLeader();
    }
    throw new IllegalStateException();
  }

  @Override
  public boolean equals(Object that) {
    if (that == null)
      return false;
    if (that instanceof LeaderResult)
      return this.equals((LeaderResult)that);
    return false;
  }

  public boolean equals(LeaderResult that) {
    if (that == null)
      return false;

    boolean this_present_projection = true && this.isSetProjection();
    boolean that_present_projection = true && that.isSetProjection();
    if (this_present_projection || that_present_projection) {
      if (!(this_present_projection && that_present_projection))
        return false;
      if (!this.projection.equals(that.projection))
        return false;
    }

    boolean this_present_isLeader = true;
    boolean that_present_isLeader = true;
    if (this_present_isLeader || that_present_isLeader) {
      if (!(this_present_isLeader && that_present_isLeader))
        return false;
      if (this.isLeader != that.isLeader)
        return false;
    }

    return true;
  }

  @Override
  public int hashCode() {
    return 0;
  }

  @Override
  public int compareTo(LeaderResult other) {
    if (!getClass().equals(other.getClass())) {
      return getClass().getName().compareTo(other.getClass().getName());
    }

    int lastComparison = 0;

    lastComparison = Boolean.valueOf(isSetProjection()).compareTo(other.isSetProjection());
    if (lastComparison != 0) {
      return lastComparison;
    }
    if (isSetProjection()) {
      lastComparison = org.apache.thrift.TBaseHelper.compareTo(this.projection, other.projection);
      if (lastComparison != 0) {
        return lastComparison;
      }
    }
    lastComparison = Boolean.valueOf(isSetIsLeader()).compareTo(other.isSetIsLeader());
    if (lastComparison != 0) {
      return lastComparison;
    }
    if (isSetIsLeader()) {
      lastComparison = org.apache.thrift.TBaseHelper.compareTo(this.isLeader, other.isLeader);
      if (lastComparison != 0) {
        return lastComparison;
      }
    }
    return 0;
  }

  public _Fields fieldForId(int fieldId) {
    return _Fields.findByThriftId(fieldId);
  }

  public void read(org.apache.thrift.protocol.TProtocol iprot) throws org.apache.thrift.TException {
    schemes.get(iprot.getScheme()).getScheme().read(iprot, this);
  }

  public void write(org.apache.thrift.protocol.TProtocol oprot) throws org.apache.thrift.TException {
    schemes.get(oprot.getScheme()).getScheme().write(oprot, this);
  }

  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder("LeaderResult(");
    boolean first = true;

    sb.append("projection:");
    if (this.projection == null) {
      sb.append("null");
    } else {
      sb.append(this.projection);
    }
    first = false;
    if (!first) sb.append(", ");
    sb.append("isLeader:");
    sb.append(this.isLeader);
    first = false;
    sb.append(")");
    return sb.toString();
  }

  public void validate() throws org.apache.thrift.TException {
    // check for required fields
    // check for sub-struct validity
    if (projection != null) {
      projection.validate();
    }
  }

  private void writeObject(java.io.ObjectOutputStream out) throws java.io.IOException {
    try {
      write(new org.apache.thrift.protocol.TCompactProtocol(new org.apache.thrift.transport.TIOStreamTransport(out)));
    } catch (org.apache.thrift.TException te) {
      throw new java.io.IOException(te);
    }
  }

  private void readObject(java.io.ObjectInputStream in) throws java.io.IOException, ClassNotFoundException {
    try {
      // it doesn't seem like you should have to do this, but java serialization is wacky, and doesn't call the default constructor.
      __isset_bitfield = 0;
      read(new org.apache.thrift.protocol.TCompactProtocol(new org.apache.thrift.transport.TIOStreamTransport(in)));
    } catch (org.apache.thrift.TException te) {
      throw new java.io.IOException(te);
    }
  }

  private static class LeaderResultStandardSchemeFactory implements SchemeFactory {
    public LeaderResultStandardScheme getScheme() {
      return new LeaderResultStandardScheme();
    }
  }

  private static class LeaderResultStandardScheme extends StandardScheme<LeaderResult> {

    public void read(org.apache.thrift.protocol.TProtocol iprot, LeaderResult struct) throws org.apache.thrift.TException {
      org.apache.thrift.protocol.TField schemeField;
      iprot.readStructBegin();
      while (true)
      {
        schemeField = iprot.readFieldBegin();
        if (schemeField.type == org.apache.thrift.protocol.TType.STOP) { 
          break;
        }
        switch (schemeField.id) {
          case 1: // PROJECTION
            if (schemeField.type == org.apache.thrift.protocol.TType.STRUCT) {
              struct.projection = new LeaderProjection();
              struct.projection.read(iprot);
              struct.setProjectionIsSet(true);
            } else { 
              org.apache.thrift.protocol.TProtocolUtil.skip(iprot, schemeField.type);
            }
            break;
          case 2: // IS_LEADER
            if (schemeField.type == org.apache.thrift.protocol.TType.BOOL) {
              struct.isLeader = iprot.readBool();
              struct.setIsLeaderIsSet(true);
            } else { 
              org.apache.thrift.protocol.TProtocolUtil.skip(iprot, schemeField.type);
            }
            break;
          default:
            org.apache.thrift.protocol.TProtocolUtil.skip(iprot, schemeField.type);
        }
        iprot.readFieldEnd();
      }
      iprot.readStructEnd();

      // check for required fields of primitive type, which can't be checked in the validate method
      struct.validate();
    }

    public void write(org.apache.thrift.protocol.TProtocol oprot, LeaderResult struct) throws org.apache.thrift.TException {
      struct.validate();

      oprot.writeStructBegin(STRUCT_DESC);
      if (struct.projection != null) {
        oprot.writeFieldBegin(PROJECTION_FIELD_DESC);
        struct.projection.write(oprot);
        oprot.writeFieldEnd();
      }
      oprot.writeFieldBegin(IS_LEADER_FIELD_DESC);
      oprot.writeBool(struct.isLeader);
      oprot.writeFieldEnd();
      oprot.writeFieldStop();
      oprot.writeStructEnd();
    }

  }

  private static class LeaderResultTupleSchemeFactory implements SchemeFactory {
    public LeaderResultTupleScheme getScheme() {
      return new LeaderResultTupleScheme();
    }
  }

  private static class LeaderResultTupleScheme extends TupleScheme<LeaderResult> {

    @Override
    public void write(org.apache.thrift.protocol.TProtocol prot, LeaderResult struct) throws org.apache.thrift.TException {
      TTupleProtocol oprot = (TTupleProtocol) prot;
      BitSet optionals = new BitSet();
      if (struct.isSetProjection()) {
        optionals.set(0);
      }
      if (struct.isSetIsLeader()) {
        optionals.set(1);
      }
      oprot.writeBitSet(optionals, 2);
      if (struct.isSetProjection()) {
        struct.projection.write(oprot);
      }
      if (struct.isSetIsLeader()) {
        oprot.writeBool(struct.isLeader);
      }
    }

    @Override
    public void read(org.apache.thrift.protocol.TProtocol prot, LeaderResult struct) throws org.apache.thrift.TException {
      TTupleProtocol iprot = (TTupleProtocol) prot;
      BitSet incoming = iprot.readBitSet(2);
      if (incoming.get(0)) {
        struct.projection = new LeaderProjection();
        struct.projection.read(iprot);
        struct.setProjectionIsSet(true);
      }
      if (incoming.get(1)) {
        struct.isLeader = iprot.readBool();
        struct.setIsLeaderIsSet(true);
      }
    }
  }

}

