/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
body
{
    font-size: 16px;
    line-height: 22px;
}

tt
{
    font-size: 14px;
    background-color: transparent;
}

pre
{
    font-size: 14px;
}

.well, .breadcrumb
{
    background-color: #f5f5ff;
}

a:link, a:visited, a:active
{
    color: #0035AC;
}

a:hover
{
    color: #001F67;
}

.nav-list .active a:link, .nav-list .active a:visited, .nav-list .active a:active, .nav-list .active a:hover
{
    background-color: #0035AC;
}

.nav-list li a:hover
{
    background-color: #DDDDDD;
}

footer
{
    font-size: 12px;
    line-height: 15px;
}

h2
{
    text-align: center;
    border-bottom: 1px solid #0035AC;
    margin-bottom: 18px;
}