/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.curator.framework.imps;

import org.apache.curator.framework.CuratorFramework;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

class FailedDeleteManager
{
    private final Logger log = LoggerFactory.getLogger(getClass());
    private final CuratorFramework client;
    
    volatile FailedDeleteManagerListener debugListener = null;
    
    interface FailedDeleteManagerListener
    {
       public void pathAddedForDelete(String path);
    }

    FailedDeleteManager(CuratorFramework client)
    {
        this.client = client;
    }

    void addFailedDelete(String path)
    {
        if ( debugListener != null )
        {
            debugListener.pathAddedForDelete(path);
        }
        
        
        if ( client.getState() == CuratorFrameworkState.STARTED )
        {
            log.debug("Path being added to guaranteed delete set: " + path);
            try
            {
                client.delete().guaranteed().inBackground().forPath(path);
            }
            catch ( Exception e )
            {
                addFailedDelete(path);
            }
        }
    }
}
